package com.jiuqi.sql;

import java.util.Date;

public abstract class OutputDataSet
{
    private boolean active;
    private FieldInfoList fieldInfos;
    private OutputRecordBuffer buffer;
    
    private FieldInfo findField(final String fieldName) throws DBException {
        final int index = this.fieldInfos.find(fieldName);
        if (index < 0) {
            throw new DBException(String.valueOf(fieldName) + " Not Exists");
        }
        final FieldInfo result = this.fieldInfos.get(index);
        return result;
    }
    
    public FieldInfo addField(final String name, final int type) {
        final FieldInfo fieldInfo = this.fieldInfos.add();
        fieldInfo.name = name;
        switch (fieldInfo.type = type) {
            case 1: {
                fieldInfo.size = 1;
                break;
            }
            case 3: {
                fieldInfo.size = 8;
                break;
            }
            case 5: {
                fieldInfo.size = 4;
                break;
            }
            case 8: {
                fieldInfo.size = 8;
                break;
            }
            case 2: {
                fieldInfo.size = 8;
                break;
            }
        }
        return fieldInfo;
    }
    
    public void addField(final String name, final int type, final int size) {
        final FieldInfo fieldInfo = this.fieldInfos.add();
        fieldInfo.name = name;
        fieldInfo.type = type;
        fieldInfo.size = size;
    }
    
    public FieldInfoList getFields() {
        return this.fieldInfos;
    }
    
    public abstract byte[] getBytes();
    
    protected abstract void postRecord(final OutputRecordBuffer p0) throws DBException;
    
    protected abstract void internalCreate(final FieldInfoList p0) throws DBException;
    
    protected abstract OutputRecordBuffer internalOpen() throws DBException;
    
    protected abstract void internalClose() throws DBException;
    
    public OutputDataSet() {
        this.fieldInfos = new FieldInfoList();
    }
    
    public void createTable() throws DBException {
        this.internalCreate(this.fieldInfos);
    }
    
    public void open() throws DBException {
        if (this.active) {
            return;
        }
        this.buffer = this.internalOpen();
        this.fieldInfos.refreshFinder();
        this.active = true;
    }
    
    public void append() throws DBException {
        this.buffer.clearBuffer();
    }
    
    public void post() throws DBException {
        this.postRecord(this.buffer);
    }
    
    public boolean isActive() {
        return this.active;
    }
    
    public void close() throws DBException {
        if (!this.active) {
            return;
        }
        this.internalClose();
        this.active = false;
    }
    
    public int fieldIndexOf(final String fieldName) {
        return this.fieldInfos.find(fieldName);
    }
    
    public void setBoolean(final int index, final boolean value) throws DBException {
        this.buffer.writeBoolean(this.fieldInfos.get(index - 1), value);
    }
    
    public void setBoolean(final String fieldName, final boolean value) throws DBException {
        this.buffer.writeBoolean(this.findField(fieldName), value);
    }
    
    public void setInt(final int index, final int value) throws DBException {
        this.buffer.writeInt(this.fieldInfos.get(index - 1), value);
    }
    
    public void setInt(final String fieldName, final int value) throws DBException {
        this.buffer.writeInt(this.findField(fieldName), value);
    }
    
    public void setLong(final int index, final long value) throws DBException {
        this.buffer.writeLong(this.fieldInfos.get(index - 1), value);
    }
    
    public void setLong(final String fieldName, final long value) throws DBException {
        this.buffer.writeLong(this.findField(fieldName), value);
    }
    
    public void setString(final int index, final String value) throws DBException {
        this.buffer.writeString(this.fieldInfos.get(index - 1), value);
    }
    
    public void setString(final String fieldName, final String value) throws DBException {
        this.buffer.writeString(this.findField(fieldName), value);
    }
    
    public void setDouble(final int index, final double value) throws DBException {
        this.buffer.writeDouble(this.fieldInfos.get(index - 1), value);
    }
    
    public void setDouble(final String fieldName, final double value) throws DBException {
        this.buffer.writeDouble(this.findField(fieldName), value);
    }
    
    public void setDate(final int index, final Date value) throws DBException {
        this.buffer.writeDate(this.fieldInfos.get(index - 1), value);
    }
    
    public void setDate(final String fieldName, final Date value) throws DBException {
        this.buffer.writeDate(this.findField(fieldName), value);
    }
    
    public void setNull(final int index) throws DBException {
        this.buffer.writeNull(this.fieldInfos.get(index - 1));
    }
    
    public void setNull(final String fieldName) throws DBException {
        this.buffer.writeNull(this.findField(fieldName));
    }
}
